@@ -34,9 +34,6 @@ struct _GnlPadPrivate
   GstPadDirection dir;
   GstPadEventFunction eventfunc;
   GstPadQueryFunction queryfunc;
-  GstPadLinkFunction linkfunc;
-  GstPadUnlinkFunction unlinkfunc;
-
 };
 
 GST_BOILERPLATE (GnlObject, gnl_object, GstBin, GST_TYPE_BIN);
@@ -147,10 +144,11 @@ gnl_object_class_init (GnlObjectClass * klass)
 
   g_object_class_install_property (gobject_class, ARG_PRIORITY,
       g_param_spec_uint ("priority", "Priority",
-          "The priority of the object", 0, G_MAXUINT, 0, G_PARAM_READWRITE));
+          "The priority of the object (0 = highest priority)", 0, G_MAXUINT, 0,
+          G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, ARG_ACTIVE,
-      g_param_spec_boolean ("active", "Active",
-          "Render this object", TRUE, G_PARAM_READWRITE));
+      g_param_spec_boolean ("active", "Active", "Render this object", TRUE,
+          G_PARAM_READWRITE));
 
   g_object_class_install_property (gobject_class, ARG_CAPS,
       g_param_spec_boxed ("caps", "Caps",
@@ -540,7 +538,7 @@ internalpad_event_function (GstPad * internal, GstEvent * event)
 {
   GnlPadPrivate *priv = gst_pad_get_element_private (internal);
   GnlObject *object = priv->object;
-  GstMessage * message = NULL;
+  GstMessage *message = NULL;
   gboolean res;
 
   GST_DEBUG_OBJECT (internal, "event:%s", GST_EVENT_TYPE_NAME (event));
@@ -555,9 +553,8 @@ internalpad_event_function (GstPad * internal, GstEvent * event)
     case GST_PAD_SRC:
       if (GST_EVENT_TYPE (event) == GST_EVENT_NEWSEGMENT) {
         event = translate_outgoing_new_segment (object, event);
-	message = gst_message_new_segment_start (GST_OBJECT (object),
-						 GST_FORMAT_TIME,
-						 (gint64) object->start);
+        message = gst_message_new_segment_start (GST_OBJECT (object),
+            GST_FORMAT_TIME, (gint64) object->start);
       }
 
       break;
@@ -637,26 +634,12 @@ internalpad_query_function (GstPad * internal, GstQuery * query)
   return priv->queryfunc (internal, query);
 }
 
-static void
-internalpad_unlink_function (GstPad * internal)
-{
-  GnlPadPrivate *priv = gst_pad_get_element_private (internal);
-
-  GST_DEBUG_OBJECT (internal, "unlinking");
-
-  if (priv->unlinkfunc)
-    priv->unlinkfunc (internal);
-  else
-    GST_WARNING_OBJECT (internal,
-        "priv->unlinkfunc == NULL !! What's going on ?");
-}
-
-
 static gboolean
 ghostpad_event_function (GstPad * ghostpad, GstEvent * event)
 {
   GnlPadPrivate *priv;
   GnlObject *object;
+  gboolean ret = FALSE;
 
   priv = gst_pad_get_element_private (ghostpad);
   object = priv->object;
@@ -677,8 +660,12 @@ ghostpad_event_function (GstPad * ghostpad, GstEvent * event)
       break;
   }
 
-  GST_DEBUG_OBJECT (ghostpad, "Calling priv->eventfunc %p", priv->eventfunc);
-  return priv->eventfunc (ghostpad, event);
+  GST_DEBUG_OBJECT (ghostpad, "Calling priv->eventfunc");
+  ret = priv->eventfunc (ghostpad, event);
+  GST_DEBUG_OBJECT (ghostpad, "Returned from calling priv->eventfunc : %d",
+      ret);
+
+  return ret;
 
   /* ERRORS */
 no_function:
@@ -714,16 +701,31 @@ ghostpad_query_function (GstPad * ghostpad, GstQuery * query)
   return pret;
 }
 
+/* internal pad going away */
+static void
+internal_pad_finalizing (GnlPadPrivate * priv, GObject * pad)
+{
+  g_free (priv);
+}
+
 static void
 control_internal_pad (GstPad * ghostpad, GnlObject * object)
 {
   GnlPadPrivate *priv;
   GnlPadPrivate *privghost = gst_pad_get_element_private (ghostpad);
-  GstPad *internal = gst_pad_get_peer (gst_ghost_pad_get_target
-      (GST_GHOST_PAD (ghostpad)));
+  GstPad *target = gst_ghost_pad_get_target (GST_GHOST_PAD (ghostpad));
+  GstPad *internal;
+
+  if (!target) {
+    GST_DEBUG_OBJECT (ghostpad,
+        "ghostpad doesn't have a target, no need to control the internal pad");
+    return;
+  }
 
   GST_LOG_OBJECT (ghostpad, "overriding ghostpad's internal pad function");
 
+  internal = gst_pad_get_peer (target);
+
   if (!(priv = gst_pad_get_element_private (internal))) {
     GST_DEBUG_OBJECT (internal,
         "Creating a GnlPadPrivate to put in element_private");
@@ -732,17 +734,16 @@ control_internal_pad (GstPad * ghostpad, GnlObject * object)
     /* Remember existing pad functions */
     priv->eventfunc = GST_PAD_EVENTFUNC (internal);
     priv->queryfunc = GST_PAD_QUERYFUNC (internal);
-    priv->unlinkfunc = GST_PAD_UNLINKFUNC (internal);
     gst_pad_set_element_private (internal, priv);
 
+    g_object_weak_ref ((GObject *) internal,
+        (GWeakNotify) internal_pad_finalizing, priv);
+
     /* add query/event function overrides on internal pad */
     gst_pad_set_event_function (internal,
         GST_DEBUG_FUNCPTR (internalpad_event_function));
     gst_pad_set_query_function (internal,
         GST_DEBUG_FUNCPTR (internalpad_query_function));
-    if (priv->unlinkfunc)
-      gst_pad_set_unlink_function (internal,
-          GST_DEBUG_FUNCPTR (internalpad_unlink_function));
   } else {
     GST_WARNING_OBJECT (internal,
         "internal pad already had an element_private");
@@ -754,50 +755,6 @@ control_internal_pad (GstPad * ghostpad, GnlObject * object)
   gst_object_unref (internal);
 }
 
-static GstPadLinkReturn
-ghostpad_link_function (GstPad * ghostpad, GstPad * peer)
-{
-  GnlPadPrivate *priv = gst_pad_get_element_private (ghostpad);
-  GstPadLinkReturn ret;
-
-  GST_DEBUG_OBJECT (ghostpad, "peer: %s:%s", GST_DEBUG_PAD_NAME (peer));
-
-  ret = priv->linkfunc (ghostpad, peer);
-
-  if (G_UNLIKELY (ret != GST_PAD_LINK_OK))
-    goto link_error;
-
-  GST_DEBUG_OBJECT (ghostpad,
-      "linking went ok, getting internal pad and overriding query/event functions");
-
-
-  return ret;
-
-  /* ERRORS */
-link_error:
-  {
-    GST_DEBUG_OBJECT (ghostpad, "failure linking: %d", ret);
-    return ret;
-  }
-}
-
-static void
-ghostpad_unlink_function (GstPad * ghostpad)
-{
-  GstPad *internal = gst_pad_get_peer (gst_ghost_pad_get_target
-      (GST_GHOST_PAD (ghostpad)));
-  GnlPadPrivate *priv;
-
-  if (!internal)
-    return;
-  priv = gst_pad_get_element_private (ghostpad);
-
-  GST_DEBUG_OBJECT (ghostpad, "Before calling parent unlink function");
-  if (priv->unlinkfunc)
-    priv->unlinkfunc (ghostpad);
-  GST_DEBUG_OBJECT (ghostpad, "After calling parent unlink function");
-
-}
 
 /**
  * gnl_object_ghost_pad:
@@ -838,7 +795,9 @@ gnl_object_ghost_pad_full (GnlObject * object, const gchar * name,
   }
 
 
- /* add it to element */
+  /* activate pad */
+  gst_pad_set_active (ghost, TRUE);
+  /* add it to element */
   if (!(gst_element_add_pad (GST_ELEMENT (object), ghost))) {
     GST_WARNING ("couldn't add newly created ghostpad");
     return NULL;
@@ -877,6 +836,18 @@ gnl_object_ghost_pad_no_target (GnlObject * object, const gchar * name,
   priv = g_new0 (GnlPadPrivate, 1);
   priv->dir = dir;
   priv->object = object;
+
+  /* grab/replace event/query functions */
+  GST_DEBUG_OBJECT (ghost, "Setting priv->eventfunc to %p",
+      GST_PAD_EVENTFUNC (ghost));
+  priv->eventfunc = GST_PAD_EVENTFUNC (ghost);
+  priv->queryfunc = GST_PAD_QUERYFUNC (ghost);
+
+  gst_pad_set_event_function (ghost,
+      GST_DEBUG_FUNCPTR (ghostpad_event_function));
+  gst_pad_set_query_function (ghost,
+      GST_DEBUG_FUNCPTR (ghostpad_query_function));
+
   gst_pad_set_element_private (ghost, priv);
 
   return ghost;
@@ -913,22 +884,6 @@ gnl_object_ghost_pad_set_target (GnlObject * object, GstPad * ghost,
   if (!(gst_ghost_pad_set_target (GST_GHOST_PAD (ghost), target)))
     return FALSE;
 
-  /* grab/replace event/query functions */
-  priv->linkfunc = GST_PAD_LINKFUNC (ghost);
-  priv->unlinkfunc = GST_PAD_UNLINKFUNC (ghost);
-  GST_DEBUG_OBJECT (ghost, "Setting priv->eventfunc to %p",
-      GST_PAD_EVENTFUNC (ghost));
-  priv->eventfunc = GST_PAD_EVENTFUNC (ghost);
-  priv->queryfunc = GST_PAD_QUERYFUNC (ghost);
-
-  gst_pad_set_link_function (ghost, GST_DEBUG_FUNCPTR (ghostpad_link_function));
-  gst_pad_set_unlink_function (ghost,
-      GST_DEBUG_FUNCPTR (ghostpad_unlink_function));
-  gst_pad_set_event_function (ghost,
-      GST_DEBUG_FUNCPTR (ghostpad_event_function));
-  gst_pad_set_query_function (ghost,
-      GST_DEBUG_FUNCPTR (ghostpad_query_function));
-
   if (!GST_OBJECT_IS_FLOATING (ghost))
     control_internal_pad (ghost, object);
 
@@ -1201,4 +1156,3 @@ gnl_object_change_state (GstElement * element, GstStateChange transition)
 beach:
   return ret;
 }
-
